//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  SFont
//  Copyright (C) 2003 Karl Bartel <karlb@gmx.net>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

//
// ***************************************************************************************
// inspired of the work of Karl Bartel in SFont - LGPL - http://www.linux-games.com/sfont/
// also of Ilana Rudnik for Unicode - irudnik@itsgames.com
// ***************************************************************************************

#include "Font.h"
#include "Lineo.h"

Font::Font(Lineo *app_lineo, string name, string name_surface, string ext)
{
	_AppLineo = app_lineo;
    int x, i;
    Uint32 pixel;
    Uint32 pink;

    _Name = name;

    _NameSurface[0]=name_surface+"_0."+ext;
    _NameSurface[1]=name_surface+"_1."+ext;

    // On charge les surfaces si besoins est dans screen
    // on calcul galement la position de chaque caractre pour chaque jeux de caractre (2 en tout)
    for(int font_i=0;font_i<2;font_i++){
        x = 0;
        i = 0;
        // on cr un sprite juste pour tre sr que la surface soit cre
        _AppLineo->_Screen->addSprite(_NameSurface[font_i],_NameSurface[font_i],Z_MIN);

        SDL_Surface* surface = _AppLineo->_Screen->getSpriteSurface(_NameSurface[font_i]);

        _SurfaceW[font_i]=surface->w;
        _SurfaceH[font_i]=surface->h;

        if (surface == NULL) {
          fprintf(stderr,
                  "\nErreur: Je ne peux pas initialiser la font!\n"
                  "Le pointeur de la surface %d est null",font_i);
          exit(1);
        }

        SDL_LockSurface(surface);

        pink = SDL_MapRGB(surface->format, 255, 0, 255);
        while (x < surface->w) {
    	    if (getPixel(surface, x, 0) == (Sint32)pink) {
                _CharPos[font_i][i++]=x;
        	    while((x < surface->w) && (getPixel(surface, x, 0)== (Sint32)pink)){
    		        x++;
                }
    	        _CharPos[font_i][i++]=x;
    	    }
    	    x++;
        }
        _MaxPos[font_i] = x-1;

        pixel = getPixel(surface, 0, surface->h-1);
        SDL_UnlockSurface(surface);
        SDL_SetColorKey(surface, SDL_SRCCOLORKEY, pixel);

        // maintenant qu'on a plus besoin de la surface on supprime le sprite cr juste avant
        _AppLineo->_Screen->delSprite(_NameSurface[font_i]);
    }
}

// constructeur utilis pour les finds
Font::Font(string name)
{
    _Name = name;
}

Font::~Font()
{
}

int Font::getPixel(SDL_Surface *Surface, Sint32 X, Sint32 Y)
{
   Uint8  *bits;
   Uint32 Bpp;

   if(X>=Surface->w) return -1;

   Bpp = Surface->format->BytesPerPixel;
   bits = ((Uint8 *)Surface->pixels)+Y*Surface->pitch+X*Bpp;

   // Get the pixel
   switch(Bpp) {
      case 1:
         return *((Uint8 *)Surface->pixels + Y * Surface->pitch + X);
         break;
      case 2:
         return *((Uint16 *)Surface->pixels + Y * Surface->pitch/2 + X);
         break;
      case 3: { // Format/endian independent
         Uint8 r, g, b;
         r = *((bits)+Surface->format->Rshift/8);
         g = *((bits)+Surface->format->Gshift/8);
         b = *((bits)+Surface->format->Bshift/8);
         return SDL_MapRGB(Surface->format, r, g, b);
         }
         break;
      case 4:
         return *((Uint32 *)Surface->pixels + Y * Surface->pitch/4 + X);
         break;
   }

   return -1;
}

// transforme une phrase en autant d'image qu'il y a de lettres
void Font::phraseToImages(string phrase, Uint32 id_phrase, SDL_Rect rect, Uint32 z_index)
{
	// on cr les 2 sprites appropris, et on indique que _noCurrent doit tre  true
	// sinon seul la dernire lettre sera affiche...
	// le nom des sprites est compos du nom de la surface+la phrase+l'unique_id afin d'tre sr
	// de ne pas avoir deux fois le mme sprite et d'craser des donnes
	_AppLineo->_Screen->addSprite(_NameSurface[0]+phrase+to_string(id_phrase),_NameSurface[0],z_index,true);
	_AppLineo->_Screen->addSprite(_NameSurface[1]+phrase+to_string(id_phrase),_NameSurface[1],z_index,true);

    int ofs;
    int i=0;
    int FontIndex=0;
    int CharIndex=33;
    const char *text=phrase.c_str();
    SDL_Rect srcrect, destrect;
    Uint16 x=rect.x;
    Uint16 y=rect.y;

    while( text[i] != L'\0' ) {
        getCharIndex((Uchar)text[i], &FontIndex, &CharIndex);

	    if( text[i] == L' ' ) {
	        x += _CharPos[0][2] - _CharPos[0][1];
        } else {
			if( FontIndex <= 1 ) {
		    	ofs = ((CharIndex - 33) * 2) + 1;
			}else {
		    	ofs = (CharIndex * 2) + 1;
			}

            // nouvelle mhode de calcul optimise (par rapport  SFont original), c'est sr !!!
            srcrect.w = destrect.w = _CharPos[FontIndex][ofs+1] - _CharPos[FontIndex][ofs];
            srcrect.h = destrect.h = _SurfaceH[FontIndex] - 1;
            srcrect.x = _CharPos[FontIndex][ofs];
            srcrect.y = 1;
            destrect.x = x;
            destrect.y = y;

            // on ajoute l'image (les coordonnes de la lettre)
            _AppLineo->_Screen->addSpriteImage(_NameSurface[FontIndex]+phrase+to_string(id_phrase),srcrect,destrect);
            x += srcrect.w;
        }
        i++;
    }
}

// dplace un texte selon un offset
void Font::movPhrase(string phrase, Uint32 id_phrase, Sint32 offset_x, Sint32 offset_y)
{
    // on dplace les sprites selon l'offset tout simplement ;-)
    _AppLineo->_Screen->movSpriteAllImages(_NameSurface[0]+phrase+to_string(id_phrase),offset_x,offset_y);
	_AppLineo->_Screen->movSpriteAllImages(_NameSurface[1]+phrase+to_string(id_phrase),offset_x,offset_y);
}

// modifie le z_index d'une phrase
void Font::setPhraseZIndex(string phrase, Uint32 id_phrase, Uint32 new_z_index)
{
    // on modifie le z_index des sprites tout simplement ;-)
    _AppLineo->_Screen->setSpriteZIndex(_NameSurface[0]+phrase+to_string(id_phrase),new_z_index);
	_AppLineo->_Screen->setSpriteZIndex(_NameSurface[1]+phrase+to_string(id_phrase),new_z_index);
}

// supprime une phrase
void Font::delPhrase(string phrase, Uint32 id_phrase)
{
    // on supprime les sprites tout simplement ;-)
    _AppLineo->_Screen->delSprite(_NameSurface[0]+phrase+to_string(id_phrase));
	_AppLineo->_Screen->delSprite(_NameSurface[1]+phrase+to_string(id_phrase));
}

// donne la largeur d'une phrase en pixel
Uint16 Font::getPhraseWidth(string phrase)
{
    int x=0,i=0;
    int ofs = 0;
    int FontIndex=0;
    int CharIndex=33;

    while( phrase[i] != L'\0' ) {
	    getCharIndex(phrase[i], &FontIndex, &CharIndex);

		if( phrase[i] == L' ' ) {
	        x += _CharPos[0][2] - _CharPos[0][1];
        } else {
			if( FontIndex <= 1 ) {
		    	ofs = ((CharIndex - 33) * 2) + 1;
			}else {
		    	ofs = (CharIndex * 2) + 1;
			}

            x += _CharPos[FontIndex][ofs+1] - _CharPos[FontIndex][ofs];
        }
        i++;
    }
    return x;
}

// donne la hauteur d'une phrase en pixel
Uint16 Font::getHeight()
{
    return (_SurfaceH[0] - 1);
}

// renvoi l'index de l'image  utilise 0|1 soit
// pour les images [nom de la font]_0
//                 [nom de la font]_1
void Font::getCharIndex(Uchar inchar, int *fontindex, int *charindex)
{
    *charindex = inchar & 0x00ff;
//	*fontindex = ((inchar >> 8) & 0x00ff) * 2;
    (*fontindex) = 0;

	if( *charindex >= 128 ) {
		(*charindex) -= 128;
		(*fontindex) = 1;
	}

}
