//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA



#ifndef __JOUEUR_H__
#define __JOUEUR_H__

#include <string>

#include <SDL.h>

#include "LaDonne.h"
#include "VueGame.h"
#include "JoueurIA.h"

#ifndef Nokia770
    #include "JoueurNet.h"
#endif

#define JOUEUR_DROITE 0
#define JOUEUR_GAUCHE 1

#define JOUEUR_TYPE_HUMAIN 0
#define JOUEUR_TYPE_IA     1
#ifndef Nokia770
    #define JOUEUR_TYPE_RESO   2
#endif

class Lineo;

using namespace std;

// Cette classe est celle qui est gre par le jeu, c'est elle seule qui fait
// la diffrence entre un joueur humain, IA ou rseau
class Joueur
{
  //*******************
  //* variables membres
	private:
        Lineo *_AppLineo;

        Uint8       _Type;
        string      _Nom;
        Uint8       _Place;

        JoueurIA  *_JoueurIA;
#ifndef Nokia770
		JoueurNet *_JoueurNet;
#endif
		bool      _NomIsSet;

  //*******************
  //* fonctions membres
	public:
		Joueur(Lineo* app_lineo, Uint8 type, string nom, Uint8 place);
		Joueur(Lineo* app_lineo, Uint8 type, Uint8 place);
		~Joueur();

		void pieceFirstWait(VueGame *vue_game, LaDonne donne);
		void pieceWait(VueGame *vue_game, LaDonne donne);
        void cercleWait(VueGame *vue_game, LaDonne donne);
        Uint8 getQuiCommence();
        void setNom(string nom);
        void processEvents(SDL_Event *event);

        inline bool isHuman(){
			return _Type==JOUEUR_TYPE_HUMAIN;
		}
        inline string getNom(){
			return _Nom;
		}
        inline bool getNomIsSet(){
			return _NomIsSet;
		}
        inline Uint8 getPlace(){
			return _Place;
		}

#ifndef Nokia770
		inline void pieceSelected(Uint8 autour_i){
             if(_JoueurNet!=NULL) _JoueurNet->pieceSelected(autour_i);
        }
        inline void cercleSelected(Uint8 i, Uint8 j){
            if(_JoueurNet!=NULL) _JoueurNet->cercleSelected(i, j, false, "");
        }
        inline void cercleSelected(Uint8 i, Uint8 j, string partie){
             if(_JoueurNet!=NULL) _JoueurNet->cercleSelected( i,  j,  true,  partie);
        }
        inline void confirmePartieEnd(){
            if(_JoueurNet!=NULL) _JoueurNet->confirmePartieEnd();
        }
        inline void newPartieWait(){
            if(_JoueurNet!=NULL) _JoueurNet->newPartieWait();
        }
#endif

    private:
  		void init(Lineo* app_lineo, Uint8 type, bool set_nom, string nom, Uint8 place);
};

#endif // __JOUEUR_H__
