//  Lino
//  Copyright (C) 2005 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA



#ifndef __LADONNE_H__
#define __LADONNE_H__

#include <SDL.h>

#include "Piece.h"

// reprsente la donne d'un joueur au moment o il doit jouer
// permet de grer :
//    - les pices libres
//    - les pices sur le plateau
//    - la pice  poser
//    - la dernire pice pose
class LaDonne
{
  //*******************
  //* variables membres
	private:
        Uint8 _Libres[16]; // liste des pices libres en dehors du plateau
        Uint8 _NbLibres; // le nombre de pices libres
        Uint8 _Choisie; // la pice qui est en attente d'tre place
        Uint8 _LastPlacee; // la dernire pice place sur le plateau
        Uint8 _Plateau[4][4]; // tableau reprsentant le plateau de 4x4 et les formes qui y sont
        
        Uint8 _LineoForme; // forme qui ralise un Lineo (P_NO_FORME si pas de Lineo!)
        Uint8 _LineoI[4]; // position en I des quatres pices du Lineo
        Uint8 _LineoJ[4]; // position en J des quatres pices du Lineo
        bool  _Lineo; // indique s'il y a ou non un Lineo
        
        bool _CarresPossibles; // indique si l'on prend en charge ou non les linos carrs
        
  //*******************
  //* fonctions membres
	public:
		LaDonne();
		~LaDonne();

		void choisirPiece(Uint8 i); // choisie une pice dans les libres
		Uint8 placerPiece(Uint8 i, Uint8 j); // place la pice choisie sur le plateau

        inline void setCarresPossibles(){ // permet de prendre en compte les carrs comme lino !
            _CarresPossibles=true;
        }

		Uint8 getPieceLibre(Uint8 i);
		
		inline bool isLineo(){ return _Lineo; }
		inline bool isEgalite(){ return (!_Lineo && _NbLibres==0); }
		inline bool isCercleLibre(Uint8 i, Uint8 j){ return _Plateau[i][j]==P_NO_FORME; }
		inline Uint8 getPlateau(Uint8 i, Uint8 j){ return _Plateau[i][j]; }
        inline Uint8 nbLibres(){ return _NbLibres; }
		inline Uint8 getPieceChoisie(){ return _Choisie; }
		inline Uint8 getLineoI(Uint8 index_i){ return _LineoI[index_i]; }
		inline Uint8 getLineoJ(Uint8 index_j){ return _LineoJ[index_j]; }

    private:
        Uint8 testLineo(Uint8 i, Uint8 j);
   		Uint8 testLineoLigne(Uint8 piece_1, Uint8 piece_2, Uint8 piece_3, Uint8 piece_4);
};

#endif // __LADONNE_H__
