//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA



#ifndef __Lineo_H__
#define __Lineo_H__

#include <vector>
#include <string>

#include <SDL.h>
#include <SDL_image.h>

#ifdef __WIN32
	#include "windows.h"
	#include "ShellApi.h"
#endif

#include "Defines.h"
#include "Screen.h"
#include "VFont.h"
#include "VTexte.h"
#include "Curseur.h"
#include "Events.h"
#include "Config.h"
#include "Timer.h"
#include "Langue.h"
#include "PopUp.h"

#include "GeneriqueDebut.h"
#include "GeneriqueFin.h"
#include "VueMainMenu.h"
#include "VueGame.h"
#include "VueNiveau.h"
#include "VueOption.h"
#include "VueSousOption.h"
#include "VueConstruction.h"
#include "VueChoixJouer.h"
#include "VueChoixLangue.h"
#include "VueChoixCouleurPiece.h"
#include "VueCredits.h"
#include "Musique.h"

#ifndef Nokia770
    #include "VueLogin.h"
    #include "VueChoixModeVideo.h"
    #include "VueChoixAdversaire.h"
    #include "ComReseau.h"
#endif

using namespace std;

enum enum_Vues
{
    VUE_NO_VUE,
    VUE_GENERIQUE_DEBUT,
    VUE_MAIN_MENU,
    VUE_GAME,
    VUE_NIVEAU,
    VUE_OPTION,
    VUE_CONSTRUCTION,
    VUE_SOUSOPTION,
    VUE_CHOIX_COULEUR_PIECE,
    VUE_CHOIX_LANGUE,
    VUE_CHOIX_JOUER,
    VUE_CREDITS,
    VUE_GENERIQUE_FIN,
#ifndef Nokia770
    VUE_CHOIX_MODE_VIDEO,
    VUE_LOGIN,
    VUE_CHOIXADVERSAIRE,
#endif
    VUE_MAX
};

// C'est la classe matresse, la premire appele, elle EST le logiciel !!!
class Lineo
{
  //*******************
  //* variables membres
    public:
		Config 	*_Config;
		Curseur	*_Curseur;
		Screen	*_Screen;
		VFont	*_VFont;
		VTexte	*_VTexte;
		Events  *_Events;
		Langue 	*_Langue;
#ifndef Nokia770
		string   _SessionReseau; //Contient l'identifiant de session renvoy par le serveur
                                 //Si il est  NULL, il faut se connecter pour le rcuprer
#endif

    private:
        bool _DebugMode;
        bool _ShowFps;

        GeneriqueDebut*     	_GeneriqueDebut;
        VueMainMenu*        	_VueMainMenu;
        VueGame*            	_VueGame;
        VueNiveau*          	_VueNiveau;
        VueOption*          	_VueOption;
        VueConstruction*    	_VueConstruction;
        VueSousOption*      	_VueSousOption;
        VueChoixCouleurPiece*	_VueChoixCouleurPiece;
        VueChoixLangue*      	_VueChoixLangue;
        VueChoixJouer*      	_VueChoixJouer;
        VueCredits*         	_VueCredits;
        GeneriqueFin*       	_GeneriqueFin;
#ifndef Nokia770
        VueChoixModeVideo*      _VueChoixModeVideo;
        VueChoixAdversaire*     _VueChoixAdversaire;
        VueLogin*          	    _VueLogin;
        SDL_Thread 	           *_Thread;
#endif

        PopUp*                  _PopUp;

        Uint8                   _VueActive; // indique la vue active


  //*******************
  //* fonctions membres
	public:
		Lineo(int argc, char * argv[]);
		~Lineo();

		// affiche une popup  l'cran
		void drawPopUp(Uint8 type, string titre, string texte);
		void drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt);
		void drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt);
		void drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt, string btn3_txt, Uint32 btn3_evt);
		void killPopUp(void);
		void waitUntilPopUpIsNotNull(void);

	private:
        void init(int argc, char * argv[]);
        void initVariables(void);
        void videoSetup(void);
        void setIcon(void);
        void parseArgs(int argc, char * argv[]);
        void run(void);
        bool processEvents(void);
        void freeVues(void);

#ifndef Nokia770
        static int staticUpdateUrlServeurThread( void *Param ){
            Lineo* Obj = reinterpret_cast<Lineo*>( Param );
            return Obj->updateUrlServeurThread(Param);
        }
        int updateUrlServeurThread(void *Param);
#endif
};

#endif // __Lineo_H__
