//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2005 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA



#include "Menu.h"
#include "Lineo.h"

Menu::Menu(Lineo *app_lineo, string titre, bool retour, Uint16 event_code_retour, Uint16 total_min)
{
    init(app_lineo, titre, retour, event_code_retour, total_min);
}

Menu::Menu(Lineo *app_lineo, string titre, bool retour, Uint16 event_code_retour)
{
    init(app_lineo, titre, retour, event_code_retour, 0);
}

Menu::Menu(string titre)
{
    // on initialise le titre
    _Titre=titre;
}

Menu::~Menu()
{
    if(_TitreFlag){
        // on efface le titre
        _AppLineo->_VTexte->del(_TitreId);
    }
    // on efface les items
    for(int i=0;i<_NbItem;i++){
        freeItem(i);
    }
    if(_NoItemInputOn<MENU_NB_MAX_ITEM) {
        delete(_ItemInputFond[_NoItemInputOn]);
    }
    // puis la flche de retour si besoin est
    if(_Retour){
        _AppLineo->_Screen->delSprite(_RetourSpriteName);
    }
    // puis le fond
    delete(_FondCouleur);
}

// initialise la classe, cette mthode est appele par les constructeurs
void Menu::init(Lineo *app_lineo, string titre, bool retour, Uint16 event_code_retour, Uint16 total_min)
{
	_AppLineo = app_lineo;
    if(titre!=MENU_SANS_TITRE){
        _TitreFlag = true;
    }else{
        _TitreFlag = false;
    }
    _TitreId = 0;
    _FlagCurseurClignotant = false;
    _MouseLeftIsDown = false;
//    _CurseurClignotantId=0;
    // on initiliase les datas pour les fleches de l'item liste
	_EventDataFlecheL=MENU_FLECHE_L;
	_EventDataFlecheR=MENU_FLECHE_R;
    // on initialise le titre
    if(_TitreFlag){
        _Titre=titre;
        _TitreRect.w = _AppLineo->_VFont->getPhraseWidth(MENU_FONT_TITRE,_Titre);
        _TitreRect.h = _AppLineo->_VFont->getHeight(MENU_FONT_TITRE);
    }
    _NbItem=0;
    // on initialise le retour s'il y a lieu
    _Retour=retour;
    if(_Retour){
	_RetourSpriteNameImg=(string)(DATA_PATH)+"menus/fleche_retour.png";
        _RetourSpriteName=_RetourSpriteNameImg+_Titre;
        _RetourEventCode=event_code_retour;
        _RetourSpriteCurrent=MENU_RETOUR_BLANC;
        _RetourRectSrcImg[MENU_RETOUR_BLANC].x=0;
        _RetourRectSrcImg[MENU_RETOUR_BLANC].y=0;
        _RetourRectSrcImg[MENU_RETOUR_BLANC].w=61;
        _RetourRectSrcImg[MENU_RETOUR_BLANC].h=60;
        _RetourRectSrcImg[MENU_RETOUR_JAUNE].x=0;
        _RetourRectSrcImg[MENU_RETOUR_JAUNE].y=60;
        _RetourRectSrcImg[MENU_RETOUR_JAUNE].w=61;
        _RetourRectSrcImg[MENU_RETOUR_JAUNE].h=60;
        _RetourRectDstImg.w=61;
        _RetourRectDstImg.h=60;
    }
    _NoItemInputOn = MENU_NB_MAX_ITEM;
    _TotalMin = total_min;
    _TotalMin = 500;
    // par defaut on centre le menu
    _AlignBottomFlag = false;
    _AlignBottomOffset = 0;
    // on init le fond
    _FondCouleur = NULL;
}

Uint8 Menu::addItem(string titre_item, Uint16 event_code){
    vector<string> liste_vide;
    liste_vide.clear();
    return addItem(titre_item, event_code, NULL, MENU_TYPE_LIEN, MENU_EVENT_NO_DATA, ITEM_SANS_IMAGE, 0, 0, liste_vide, 0, 0, false);
}

//  rserver pour le type MENU_TYPE_INPUT
Uint8 Menu::addItem(string titre_item, Uint16 event_code, string *value_input){
    vector<string> liste_vide;
    liste_vide.clear();
    return addItem(titre_item, event_code, value_input, MENU_TYPE_INPUT, MENU_EVENT_NO_DATA, ITEM_SANS_IMAGE, 0, 0, liste_vide, 0, 0, false);
}

//  rserver pour le type MENU_TYPE_INPUT avec la possibilit de cacher les lettres du mot
Uint8 Menu::addItem(string titre_item, Uint16 event_code, string *value_input, bool is_password){
    vector<string> liste_vide;
    liste_vide.clear();
    return addItem(titre_item, event_code, value_input, MENU_TYPE_INPUT, MENU_EVENT_NO_DATA, ITEM_SANS_IMAGE, 0, 0, liste_vide, 0, 0, is_password);
}

Uint8 Menu::addItem(string titre_item, Uint16 event_code, Uint8 type){
    vector<string> liste_vide;
    liste_vide.clear();
    return addItem(titre_item, event_code, NULL, type, MENU_EVENT_NO_DATA, ITEM_SANS_IMAGE, 0, 0, liste_vide, 0, 0, false);
}

Uint8 Menu::addItem(string titre_item, Uint16 event_code, Uint8 type, Uint32 data){
    vector<string> liste_vide;
    liste_vide.clear();
    return addItem(titre_item, event_code, NULL, type, data, ITEM_SANS_IMAGE, 0, 0, liste_vide, 0, 0, false);
}

Uint8 Menu::addItem(string titre_item, Uint16 event_code, Uint8 type, Uint32 data, string sprite_name, Uint16 sprite_w, Uint16 sprite_h){
    vector<string> liste_vide;
    liste_vide.clear();
    return addItem(titre_item, event_code, NULL, type, data, sprite_name, sprite_w, sprite_h, liste_vide, 0, 0, false);
}

//  rserver pour le type MENU_TYPE_TABLEAU
Uint8 Menu::addItem(string titre_item, Uint16 event_code, vector<string> tab_values, Uint16 tab_w, Uint8 nb_lignes){
    return addItem(titre_item, event_code, NULL, MENU_TYPE_TABLEAU, MENU_EVENT_NO_DATA, ITEM_SANS_IMAGE, 0, 0, tab_values, tab_w, nb_lignes, false);
}

// ajoute une ligne d'option (un item) au  menu
// si event_code == 0 >>> pas d'vnement !!!
// ATTENTION !!! data a une valeur rserve : 0xFFFF ==> MENU_EVENT_NO_DATA
Uint8 Menu::addItem(string titre_item, Uint16 event_code, string *value_input, Uint8 type, Uint32 data, string sprite_name, Uint16 sprite_w, Uint16 sprite_h, vector<string> tab_values, Uint16 tab_w, Uint8 nb_lignes, bool is_password)
{
    if(_NbItem<MENU_NB_MAX_ITEM){
        Uint8 no_item = _NbItem;
        _NbItem++;
        // on essaye de faire en sorte que le titre de cet item soit unique afin de ne pas avoir de problme
        _Item[no_item] = titre_item + to_string(no_item) + to_string(type) + sprite_name;
        // on attribut chaque variable de l'item,
		// certaines par defaut qui seront modifies dans le switch ci-dessous
        _ItemFont[no_item]=MENU_FONT_ITEM_OUT;
        _ItemType[no_item]=type;
        _ItemRect[no_item].h = _AppLineo->_VFont->getHeight(_ItemFont[no_item]);
        _ItemEventCode[no_item]=event_code;
        _ItemEventData[no_item]=data;
		_ItemSpriteName[no_item]=sprite_name;
		_ItemSpriteRect[no_item].w=sprite_w;
		_ItemSpriteRect[no_item].h=sprite_h;
		_ItemFlecheFontL[no_item]=MENU_FONT_FLECHE_OUT;
		_ItemFlecheFontR[no_item]=MENU_FONT_FLECHE_OUT;
		_ItemTitreId[no_item]=0;
		_ItemFlecheIdL[no_item]=0;
		_ItemFlecheIdR[no_item]=0;
		_ItemInputValue[no_item]=value_input;
		_ItemIsPassword[no_item]=is_password;
		_ItemTabValues[no_item]=tab_values;
        _ItemTabW[no_item]=tab_w; // utile seulement pour MENU_TYPE_TABLEAU
        _ItemTabNbLignes[no_item]=nb_lignes; // utile seulement pour MENU_TYPE_TABLEAU
        _ItemTabNumFirst[no_item]=0; // utile seulement pour MENU_TYPE_TABLEAU
        _ItemTabBtHaut[no_item]=false;
        _ItemTabBtBas[no_item]=false;
        _ItemTabLigneFond[no_item].clear();
        _ItemTabLigneNumSelected[no_item]=0;
        _ItemTabLigneSelected[no_item]=false;
        _ItemTabValueSelected[no_item]="ERREUR ==> Menu::addItem";

		// on modifie certains paramtre en fonction du type
        switch(type){
			case MENU_TYPE_RADIO:
	            _ItemTitre[no_item] = titre_item;
    	        _ItemRect[no_item].w = _AppLineo->_VFont->getPhraseWidth(_ItemFont[no_item]," "+titre_item);
        	    _ItemTypeValue[no_item]=MENU_TYPE_RADIO_OFF;
			break;
        	case MENU_TYPE_INPUT:
	            _ItemTitre[no_item] = titre_item + " :  ";
    	        _ItemRect[no_item].w = _AppLineo->_VFont->getPhraseWidth(_ItemFont[no_item],_ItemTitre[no_item]+*_ItemInputValue[no_item]);
        	    _ItemTypeValue[no_item]=MENU_TYPE_INPUT_OFF;
        	break;
			case MENU_TYPE_TABLEAU:{
			    _ItemTitre[no_item] = titre_item;
                _ItemFont[no_item]=MENU_FONT_TABLEAU;
                // on calcul les dimensions du tableau
                _ItemRect[no_item].h = _ItemTabNbLignes[no_item] * _AppLineo->_VFont->getHeight(_ItemFont[no_item]);
                _ItemRect[no_item].w = _ItemTabW[no_item];
                // on fait des calculs sur l'ascenceur
                    // on dfinit s'il y aura besoin d'un ascenceur ou non
                if(_ItemTabValues[no_item].size() > _ItemTabNbLignes[no_item]){
                    _ItemTabWithAsc[no_item] = true;
                }else{
                    _ItemTabWithAsc[no_item] = false;
                }
                if(_ItemTabWithAsc[no_item]){
                    // on calcul la hauteur de l'ascenceur
                    _ItemTabAscRect[no_item].h = _ItemRect[no_item].h - MENU_ITEM_TAB_ASC_W - MENU_ITEM_TAB_ASC_W;
                    Uint32 min = _ItemTabNbLignes[no_item];
                    Uint32 max = _ItemTabValues[no_item].size();
                    float reduc = 10.0;
                    for(Uint32 i=min;i<max && _ItemTabAscRect[no_item].h > 20;i++){
                        _ItemTabAscRect[no_item].h -= (Uint16)reduc;
                        if(reduc>1.0)reduc-=0.5;
                    }
                    // on calcul le pas de dplacement de l'ascenceur, un pas correspondant ensuite  une ligne du tableau
                    _ItemTabAscPas[no_item] = (float) ((float)(_ItemRect[no_item].h - MENU_ITEM_TAB_ASC_W - MENU_ITEM_TAB_ASC_W - _ItemTabAscRect[no_item].h) / (float)(_ItemTabValues[no_item].size() - _ItemTabNbLignes[no_item]));
                }
                // on initialise les ids qui grerons les lignes de texte  l'cran
                Uint32 size = _ItemTabValues->size();
                for(Uint8 i=0;i<size;i++){
                    Uint32 id = 0;
                    _ItemTabValuesId[no_item].push_back(id);
                }
			}break;
        	case MENU_TYPE_LISTE:{
	            _ItemTitre[no_item] = titre_item;
        	    _ItemTypeValue[no_item]=MENU_TYPE_INPUT_OFF;
				// on calcul la largeur max de l'item
				Uint16 w_image_fleches = _AppLineo->_VFont->getPhraseWidth(_ItemFlecheFontL[no_item],"") + 2 * MENU_TYPE_LISTE_MARGE_IMG + _ItemSpriteRect[no_item].w;
				Uint16 w_legende = (_ItemTitre[no_item]==ITEM_SANS_TITRE) ? 0 : _AppLineo->_VFont->getPhraseWidth(_ItemFont[no_item],_ItemTitre[no_item]);
				_ItemRect[no_item].w = Max(w_image_fleches,w_legende);
				// on calcul la hauteur max de l'item, on calcul du haut de l'image en bas du texte avec une marge sous l'image qu'il y est ou non le texte...
				_ItemRect[no_item].h = _ItemSpriteRect[no_item].h + MENU_TYPE_LISTE_MARGE_IMG +((_ItemTitre[no_item]==ITEM_SANS_TITRE) ? 0 : _AppLineo->_VFont->getHeight(_ItemFont[no_item]));
        	}break;
			case MENU_TYPE_LIEN:
			default:
            	_ItemTitre[no_item] = titre_item;
	            _ItemRect[no_item].w = _AppLineo->_VFont->getPhraseWidth(_ItemFont[no_item],titre_item);
	        break;
        }

        // puis on initialise les positions (x,y)
        initPositions();
        // et enfin on retourne le numro d'item
        return no_item;
    }
    return MENU_NB_MAX_ITEM; // nombre d'item max dpass
}

// met l'un des items radio ou input  on et tous les autres  off
void Menu::setItemOn(Uint8 no_item)
{
    // on parcourt la liste des items
    // on met tous ceux qui ont le type radio  off sauf celui pass en paramtre
    Uint8 i;
    for(i=0;i<_NbItem;i++){
        if(_ItemType[i]==MENU_TYPE_RADIO){
            if(i==no_item){
                if(_ItemTypeValue[i]!=MENU_TYPE_RADIO_ON){
                    freeItem(i);
                    _ItemTypeValue[i]=MENU_TYPE_RADIO_ON;
                    drawItem(i);
                }
            }else{
                if(_ItemTypeValue[i]!=MENU_TYPE_RADIO_OFF){
                    freeItem(i);
                    _ItemTypeValue[i]=MENU_TYPE_RADIO_OFF;
                    drawItem(i);
                }
            }
        }else if(_ItemType[i]==MENU_TYPE_INPUT){
            if(i==no_item){
                if(_ItemTypeValue[i]!=MENU_TYPE_INPUT_ON){
					// s'il s'agit d'un mot de passe, comme par scurit on affiche pas le mot de passe
					// car on ne connait que son md5, on efface automatiquement le champs ds que le user veut le modifier
					if(_ItemIsPassword[i]){
		                *_ItemInputValue[i]="";
					}
                    freeItem(i);
                    _ItemTypeValue[i]=MENU_TYPE_INPUT_ON;
                    drawItem(i);
                    _NoItemInputOn=no_item;
					_FlagCurseurClignotant=true;
					refreshCurseur();
                    _ItemInputFond[i] = new Fond(_AppLineo, FOND_SAISIE, _ItemInputFondRect[i], Z_MENU_FOND_ITEM);
                }
            }else{
                if(_ItemTypeValue[i]!=MENU_TYPE_INPUT_OFF){
                    delete(_ItemInputFond[i]);
                    freeItem(i);
                    _ItemTypeValue[i]=MENU_TYPE_INPUT_OFF;
                    drawItem(i);
                }
            }
        }
    }
}

// permet d'aligner le menu sur le bas de l'cran  l'aide d'un offset
void Menu::setAlignBottom(Uint16 offset){
    _AlignBottomFlag=true;
    _AlignBottomOffset=offset;
    // on rinitialise les positions (x,y)
    initPositions();
}

// gre la position de tous les lments du menu
// on agence tous les SDL_Rect de faon dfinitive
// afin de faciliter la tche dans la fonction drawScreen
void Menu::initPositions(void)
{
    SDL_Rect total;
    int i;
  // on calcul le w max
    total.w=0;
    for(i=0;i<_NbItem;i++){
        if(_ItemRect[i].w > total.w){
            total.w = _ItemRect[i].w;
        }
        // on en profite pour calculer la position en x de l'item
        _ItemRect[i].x = MENU_FOND_MARGE_G;
    }
    // au cas ou une largeur minimum est xige...
    if ( (total.w < _TotalMin) && ( _TotalMin > 0 ) ){
       total.w = _TotalMin;
    }
    // on vite de dpasser en largeur le max car aprs c'est pas beau :-)
    if(total.w > MENU_WIDTH_MAX){
        total.w = MENU_WIDTH_MAX;
    }
    // on en profite pour calculer la position en x de la fleche
    if(_Retour){
        _RetourRectDstImg.x = 8;
    }
  // on calcul le h total
    total.h = 0;
    for(i=0;i<_NbItem;i++){
        // on en profite pour calculer la position en y de l'item
        _ItemRect[i].y = total.h;
        // on ajoute la hauteur de l'item  la hauteur totale
        total.h += _ItemRect[i].h;
        // on ajoute la hauteur de l'interligne pour le prochain item
        total.h += MENU_IL_ITEM;
    }
    // on supprime le dernier interligne car on n'en veut pas en dessous
    // c'est plus rapidement excuter que de vrifier avec un if dans le for
    total.h -= MENU_IL_ITEM;
  // on calcul la position en x total centre
    total.x = (Sint16)((SCREEN_W - total.w) * 0.5);
  // on calcul la position en y total
    if(_AlignBottomFlag){ // on aligne vers le bas d'aprs l'offset _AlignBottomOffset
        total.y = SCREEN_H - total.h - _AlignBottomOffset;
    }else{ // on centre le menu en y
        total.y = (Sint16)((SCREEN_H - total.h - MENU_MARGE_LOGO) * 0.5 + MENU_MARGE_LOGO);
    }
  // maintenant on applique l'offset pour le y de chacun, qui correspond  la pos en y totale
    if(_TitreFlag){
        _TitreRect.y += total.y;
    }
    for(i=0;i<_NbItem;i++){
        _ItemRect[i].y += total.y;
    }
    if(_Retour){
        _RetourRectDstImg.y += total.y;
    }
  // ===========================================================================
  // pour les MENU_TYPE_INPUT on a besoin de faire un calcul plus compliqu  cause
  // de l'alignement que l'on veut appliquer entre les diffrents MENU_TYPE_INPUT
  // tout en offrant un maximum de largeur possible au champs de saisie
  // et ceci on le fait aprs que l'on connat la largeur totale du menu
    // on adapte alors tous les MENU_TYPE_INPUT en fonction de max_input_w
    for(i=0;i<_NbItem;i++){
        if(_ItemType[i] == MENU_TYPE_INPUT){
            // on calcul le rectangle du fond blanc qui apparat lors de la saisie
            _ItemInputFondRect[i].x = _ItemRect[i].x + _AppLineo->_VFont->getPhraseWidth(_ItemFont[i],_ItemTitre[i]) - 8;
            _ItemInputFondRect[i].y = _ItemRect[i].y - 3;
            _ItemInputFondRect[i].w = 800 - _ItemInputFondRect[i].x - 4;
            _ItemInputFondRect[i].h = 38;
        }
    }
  // ===========================================================================
  // pour les tableaux il faut galement faire quelques calculs pour le
  // fond entre autre
    for(i=0;i<_NbItem;i++){
        if(_ItemType[i] == MENU_TYPE_TABLEAU){
            _ItemFondRect[i] = _ItemRect[i];
            _ItemFondRect[i].w = _ItemFondRect[i].w - MENU_ITEM_TAB_ASC_W - 2;
        }
    }
  // ===========================================================================
  // MAINTENANT les contours du fond du menu sont arrt
  // donc IL NE FAUT PLUS Y TOUCHER, dans cette fonction aprs a...
  // on initialise le fond en couleur
    // on ajoute une marge au contour pour le fond
    total.x =0;
    total.y-=MENU_FOND_MARGE_H-10;
    total.w =800;
    total.h+=MENU_FOND_MARGE_H+10;
    _ContourTotal = total;
  // on initialise quelques trucs pour les MENU_TYPE_LISTE
    for(i=0;i<_NbItem;i++){
        if(_ItemType[i]==MENU_TYPE_LISTE){
            // on initialise la position de l'image
			_ItemSpriteRect[i].x = ((Sint32)((_ItemRect[i].w-_ItemSpriteRect[i].w)/2))+_ItemRect[i].x;
			_ItemSpriteRect[i].y = _ItemRect[i].y;
			// on initialise les dimensions des fleches  gauche et  droite de l'image pour le survol de la souris
			_ItemFlecheRectL[i].w = _AppLineo->_VFont->getPhraseWidth(_ItemFlecheFontL[i],"");
			_ItemFlecheRectR[i].w = _AppLineo->_VFont->getPhraseWidth(_ItemFlecheFontR[i],"");
			_ItemFlecheRectL[i].h = _AppLineo->_VFont->getHeight(_ItemFlecheFontL[i]);
			_ItemFlecheRectR[i].h = _AppLineo->_VFont->getHeight(_ItemFlecheFontR[i]);
			// on initialise la position des fleches  gauche et  droite de l'image
			_ItemFlecheRectL[i].x = _ItemRect[i].x;
			_ItemFlecheRectR[i].x = _ItemRect[i].x + _ItemRect[i].w - _ItemFlecheRectR[i].w;
			_ItemFlecheRectL[i].y = ((Sint32)((_ItemSpriteRect[i].h-_ItemFlecheRectL[i].h)/2))+_ItemRect[i].y;
			_ItemFlecheRectR[i].y = ((Sint32)((_ItemSpriteRect[i].h-_ItemFlecheRectR[i].h)/2))+_ItemRect[i].y;
        }
    }
  // ===========================================================================
  // MAINTENANT seulement on peut calculer la position du Titre car elle dpend
  // de la bande blanche du fond du menu
    if(_TitreFlag){
        _TitreRect.x = 10;
        _TitreRect.y = total.y - _AppLineo->_VFont->getHeight(MENU_FONT_TITRE);
    }
    // On calcul galement la flche de retour
    if(_Retour){
        _RetourRectDstImg.y = total.y + total.h - MENU_FOND_MARGE_B - _RetourRectDstImg.h + 10;
    }
}

// affiche le menu  l'cran
void Menu::draw(void)
{
    if(_TitreFlag){
        // on cr le titre
        _TitreId = _AppLineo->_VTexte->add(MENU_FONT_TITRE,_Titre,TEXTE_X_RIGHT,_TitreRect.x,_TitreRect.y,Z_MENU_TEXTE);
    }
    // on cr les items
    for(int i=0;i<_NbItem;i++){
        drawItem(i);
    }
    // puis la flche de retour si besoin est
    if(_Retour){
        _AppLineo->_Screen->addSprite(_RetourSpriteName,_RetourSpriteNameImg,Z_MENU_TEXTE);
        _AppLineo->_Screen->addSpriteImage(_RetourSpriteName,_RetourRectSrcImg[MENU_RETOUR_BLANC],_RetourRectDstImg);
        _AppLineo->_Screen->addSpriteImage(_RetourSpriteName,_RetourRectSrcImg[MENU_RETOUR_JAUNE],_RetourRectDstImg);
        _AppLineo->_Screen->setSpriteCurrent(_RetourSpriteName,MENU_RETOUR_BLANC);
    }
    // on dessine le fond galement
    _FondCouleur = new Fond(_AppLineo, FOND_MENU, _ContourTotal, Z_MENU_FOND);
}

// affiche un item  l'cran
void Menu::drawItem(Uint8 no_item)
{
    switch(_ItemType[no_item]){
        case MENU_TYPE_RADIO:
            if(_ItemTypeValue[no_item]==MENU_TYPE_RADIO_ON){
                _ItemTitreId[no_item] = _AppLineo->_VTexte->add(_ItemFont[no_item]," "+_ItemTitre[no_item],_ItemRect[no_item].x,_ItemRect[no_item].y,Z_MENU_TEXTE);
            }else{
                _ItemTitreId[no_item] = _AppLineo->_VTexte->add(_ItemFont[no_item],_ItemTitre[no_item],_ItemRect[no_item].x,_ItemRect[no_item].y,Z_MENU_TEXTE);
            }
        break;
        case MENU_TYPE_INPUT:
            if(_ItemIsPassword[no_item]){
                // c'est un champs de saisie d'un mot de passe ou quelques chose qui doit tre cach
                // on remplace donc le texte saisi par le caractre #
                Uint8 size = (*_ItemInputValue[no_item]).size();
                string password = "";
                for(Uint8 i=0;i<size;i++){
                    password+="#";
                }
                _ItemTitreId[no_item] = _AppLineo->_VTexte->add(_ItemFont[no_item],_ItemTitre[no_item]+password,TEXTE_X_LEFT,_ItemRect[no_item].x,_ItemRect[no_item].y,Z_MENU_TEXTE);
                _ItemRect[no_item].w = _AppLineo->_VFont->getPhraseWidth(_ItemFont[no_item],_ItemTitre[no_item]+password);
            }else{
                // c'est un simple champs de saisie
                _ItemTitreId[no_item] = _AppLineo->_VTexte->add(_ItemFont[no_item],_ItemTitre[no_item]+*_ItemInputValue[no_item],TEXTE_X_LEFT,_ItemRect[no_item].x,_ItemRect[no_item].y,Z_MENU_TEXTE);
                _ItemRect[no_item].w = _AppLineo->_VFont->getPhraseWidth(_ItemFont[no_item],_ItemTitre[no_item]+*_ItemInputValue[no_item]);
            }
        break;
        case MENU_TYPE_LISTE:
			// on positionne l'image et on la rend visible
			_AppLineo->_Screen->setSpritePosXY(_ItemSpriteName[no_item],_ItemSpriteRect[no_item].x,_ItemSpriteRect[no_item].y);
			_AppLineo->_Screen->setSpriteZIndex(_ItemSpriteName[no_item],Z_MENU_TEXTE);
			// on ajoute les fleches  gauche et  droite de l'image
			_ItemFlecheIdL[no_item] = _AppLineo->_VTexte->add(_ItemFlecheFontL[no_item],"",_ItemFlecheRectL[no_item].x,_ItemFlecheRectL[no_item].y,Z_MENU_TEXTE);
			_ItemFlecheIdR[no_item] = _AppLineo->_VTexte->add(_ItemFlecheFontR[no_item],"",_ItemFlecheRectR[no_item].x,_ItemFlecheRectR[no_item].y,Z_MENU_TEXTE);
			// on ajoute le texte en dessous s'il y a lieu
			if(_ItemTitre[no_item]!=ITEM_SANS_TITRE){
                _ItemTitreId[no_item] =
                    _AppLineo->_VTexte->add(_ItemFont[no_item],
                                            _ItemTitre[no_item],
                                            (Uint32)(_ItemRect[no_item].x+_ItemRect[no_item].w*0.5-_AppLineo->_VFont->getPhraseWidth(_ItemFont[no_item],_ItemTitre[no_item])*0.5),
                                            _ItemRect[no_item].y+_ItemRect[no_item].h-_AppLineo->_VFont->getHeight(_ItemFont[no_item]),
                                            Z_MENU_TEXTE);
            }
        break;
        case MENU_TYPE_TABLEAU:{
            // on affiche d'abord le fond
            _ItemFond[no_item] = new Fond(_AppLineo, FOND_TABLEAU, _ItemFondRect[no_item], Z_MENU_FOND_ITEM);
            // puis l'ascenceur
            drawAscenceur(no_item);
            // ensuite l'affichage vari selon que la liste soit vide ou non
            if(_ItemTabValues[no_item].size() > 0){
                // puis les surlignages pour la slection d'une ligne
                // on cache les lignes en les disposant derrire le fond d'cran, astuce ;-)
                SDL_Rect fdrc; // rectangle du fond
                fdrc.x = _ItemFondRect[no_item].x;
                fdrc.y = _ItemFondRect[no_item].y;
                fdrc.w = _ItemFondRect[no_item].w;
                fdrc.h = _ItemFondRect[no_item].h / _ItemTabNbLignes[no_item];
                for(Uint8 i=0;i<_ItemTabNbLignes[no_item];i++){
                    _ItemTabLigneFond[no_item][i] = new Fond(_AppLineo, FOND_TABLEAU_LIGNE, fdrc, Z_MIN);
                    fdrc.y+=fdrc.h;
                }
                // puis les textes
                Uint16 y = _ItemRect[no_item].y;
                Uint16 h = _AppLineo->_VFont->getHeight(_ItemFont[no_item]);
                Uint8 max = Min((Uint8) (_ItemTabNumFirst[no_item]+_ItemTabNbLignes[no_item]), (Uint8) (_ItemTabValues[no_item].size()) );
                for(Uint32 i=_ItemTabNumFirst[no_item];i<max;i++){
                    _ItemTabValuesId[no_item][i] = _AppLineo->_VTexte->add(_ItemFont[no_item]," "+_ItemTabValues[no_item][i],_ItemRect[no_item].x,y,Z_MENU_TEXTE);
                    y+=h;
                }
            }else{ // la liste est vide
                // on affiche un message comme quoi la liste est vide
                Uint16 y = _ItemRect[no_item].y + (_ItemRect[no_item].h / 2 - _AppLineo->_VFont->getHeight(_ItemFont[no_item]) / 2);
                // Uint16 h = _AppLineo->_VFont->getHeight(_ItemFont[no_item]);
                _ItemTitreId[no_item] = _AppLineo->_VTexte->add(_ItemFont[no_item],_ItemTitre[no_item],TEXTE_X_CENTER,_ItemRect[no_item].x,y,Z_MENU_TEXTE);
            }
        }break;
        case MENU_TYPE_LIEN: // c'est le par defaut
        default:
            _ItemTitreId[no_item] = _AppLineo->_VTexte->add(_ItemFont[no_item],_ItemTitre[no_item],_ItemRect[no_item].x,_ItemRect[no_item].y,Z_MENU_TEXTE);
        break;
    }
}

// efface un item  l'cran
void Menu::freeItem(Uint8 no_item)
{
    switch(_ItemType[no_item]){
        case MENU_TYPE_RADIO:
            if(_ItemTypeValue[no_item]==MENU_TYPE_RADIO_ON){
                _AppLineo->_VTexte->del(_ItemTitreId[no_item]);
            }else{
                _AppLineo->_VTexte->del(_ItemTitreId[no_item]);
            }
        break;
        case MENU_TYPE_INPUT:
            if(_ItemTypeValue[no_item]==MENU_TYPE_INPUT_ON){
                _AppLineo->_VTexte->del(_ItemTitreId[no_item]);
            }else{
                _AppLineo->_VTexte->del(_ItemTitreId[no_item]);
            }
        break;
        case MENU_TYPE_LISTE:
            _AppLineo->_VTexte->del(_ItemFlecheIdL[no_item]);
            _AppLineo->_VTexte->del(_ItemFlecheIdR[no_item]);
            _AppLineo->_VTexte->del(_ItemTitreId[no_item]);
        break;
        case MENU_TYPE_TABLEAU:{
            delete(_ItemFond[no_item]);
        	_AppLineo->_Screen->delSprite(_ItemSpriteName[no_item]);
            Uint8 size = _ItemTabLigneFond[no_item].size();
            for(Uint8 i=0;i<size;i++){
                delete(_ItemTabLigneFond[no_item][i]);
            }
            _AppLineo->_VTexte->del(_ItemTitreId[no_item]);
            Uint8 max = Min((Uint8) (_ItemTabNumFirst[no_item]+_ItemTabNbLignes[no_item]), (Uint8) (_ItemTabValues[no_item].size()) );
            for(Uint32 i=_ItemTabNumFirst[no_item];i<max;i++){
                _AppLineo->_VTexte->del(_ItemTabValuesId[no_item][i]);
            }
        }break;
        case MENU_TYPE_LIEN: // c'est le par defaut
        default:
            _AppLineo->_VTexte->del(_ItemTitreId[no_item]);
        break;
    }
}

// regarde si le curseur de la souris est ou n'est pas sur l'item
// on choisit alors la Font qui s'impose en consquence
// on retourne true si on est ON et false si on est OUT
bool Menu::testItemOnOut(Uint8 no_item, Uint16 x, Uint16 y)
{
	bool on=false;
	Uint16 x1, x2, y1, y2;
	string new_font;

    switch(_ItemType[no_item]){
    	case MENU_TYPE_LISTE:{
    		// ======================================
    		// d'abord la flche gauche
    		x1=_ItemFlecheRectL[no_item].x - MENU_MARGE_ON;
    		x2=_ItemFlecheRectL[no_item].x+_ItemFlecheRectL[no_item].w+MENU_MARGE_ON+MENU_MARGE_ON;
    		y1=_ItemFlecheRectL[no_item].y-MENU_MARGE_ON;
    		y2=_ItemFlecheRectL[no_item].y+_ItemFlecheRectL[no_item].h+MENU_MARGE_ON+MENU_MARGE_ON;

    		new_font=_ItemFlecheFontL[no_item];

    		// on met la bonne font suivant si on est sur la fleche ou non
    		if(x<x1 || x>x2 || y<y1 || y>y2){
    			new_font=MENU_FONT_FLECHE_OUT;
    		}else{
    			new_font=MENU_FONT_FLECHE_ON;
    			on=true;
    		}

    		if(new_font!=_ItemFlecheFontL[no_item]){
    			freeItem(no_item);
    			_ItemFlecheFontL[no_item] = new_font;
    			drawItem(no_item);
    		}

    		// ======================================
    		// puis la flche droite
    		x1=_ItemFlecheRectR[no_item].x - MENU_MARGE_ON;
    		x2=_ItemFlecheRectR[no_item].x+_ItemFlecheRectR[no_item].w+MENU_MARGE_ON+MENU_MARGE_ON;
    		y1=_ItemFlecheRectR[no_item].y-MENU_MARGE_ON;
    		y2=_ItemFlecheRectR[no_item].y+_ItemFlecheRectR[no_item].h+MENU_MARGE_ON+MENU_MARGE_ON;

    		new_font=_ItemFlecheFontR[no_item];

    		// on met la bonne font suivant si on est sur la fleche ou non
    		if(x<x1 || x>x2 || y<y1 || y>y2){
    			new_font=MENU_FONT_FLECHE_OUT;
    		}else{
    			new_font=MENU_FONT_FLECHE_ON;
    			on=true;
    		}

    		if(new_font!=_ItemFlecheFontR[no_item]){
    			freeItem(no_item);
    			_ItemFlecheFontR[no_item] = new_font;
    			drawItem(no_item);
    		}
    	}break;

    	case MENU_TYPE_TABLEAU:{
    	    if(_ItemTabValues[no_item].size() > 0){
                if(_ItemTabWithAsc[no_item]){
                    // ======================================
                    // d'abord le bouton haut
                    x1=_ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
                    x2=_ItemFondRect[no_item].x + _ItemFondRect[no_item].w + MENU_ITEM_TAB_ASC_W + 2;
                    y1=_ItemFondRect[no_item].y;
                    y2=_ItemFondRect[no_item].y + MENU_ITEM_TAB_ASC_W;
                    // on indique si on survole le bouton du haut ou non
                    if(x<x1 || x>x2 || y<y1 || y>y2){
                        _ItemTabBtHaut[no_item] = false;
                    }else{
                        _ItemTabBtHaut[no_item] = true;
                        on=true;
                    }

                    // ======================================
                    // puis la barre de l'ascenceur
/*  @@@ Ceci est pour plus tard pour permettre de cliquer sur la barre pour la dplacer...
                    x1=_ItemFondRect[no_item].x + _ItemFondRect[no_item].w - MENU_ITEM_TAB_ASC_W;
                    x2=_ItemFondRect[no_item].x + _ItemFondRect[no_item].w;
                    y1=_ItemFondRect[no_item].y + MENU_ITEM_TAB_ASC_W;
                    y2=_ItemFondRect[no_item].y + _ItemFondRect[no_item].h - MENU_ITEM_TAB_ASC_W;
                    // on indique si on survole l'ascenceur ou non
                    if(x<x1 || x>x2 || y<y1 || y>y2){
                        _ItemTabAsc[no_item] = false;
                    }else{
                        _ItemTabAsc[no_item] = true;
                        on=true;
                    }
*/
                    // ======================================
                    // puis le bouton bas
                    x1=_ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
                    x2=_ItemFondRect[no_item].x + _ItemFondRect[no_item].w + MENU_ITEM_TAB_ASC_W + 2;
                    y1=_ItemFondRect[no_item].y + _ItemFondRect[no_item].h - MENU_ITEM_TAB_ASC_W;
                    y2=_ItemFondRect[no_item].y + _ItemFondRect[no_item].h;
                    // on indique si on survole le bouton du bas ou non
                    if(x<x1 || x>x2 || y<y1 || y>y2){
                        _ItemTabBtBas[no_item] = false;
                    }else{
                        _ItemTabBtBas[no_item] = true;
                        on=true;
                    }
                }
                // ======================================
                // puis le surlignage d'une ligne si ncessaire
                x1=_ItemFondRect[no_item].x;
                x2=_ItemFondRect[no_item].x + _ItemFondRect[no_item].w - MENU_ITEM_TAB_ASC_W - 2; // on raccourci  droite la zone cliquable pour ne pas gner l'ascenceur...
                y1=_ItemFondRect[no_item].y;
                y2=_ItemFondRect[no_item].y + _ItemFondRect[no_item].h;
                _ItemTabLigneSelected[no_item]=false;
                // on cache toutes les lignes avant de faire apparaitre l'une d'elles si ncessaire
                Uint8 size = _ItemTabLigneFond[no_item].size();
                for(Uint8 i=0;i<size;i++){
                    _ItemTabLigneFond[no_item][i]->setZIndex(Z_MIN);
                }
                // on vrifie si on survole la zone du tableau
                if(x<x1 || x>x2 || y<y1 || y>y2){
                }else{
                    // maintenant on cherche o placer la barre
                    Uint8 h = _ItemFondRect[no_item].h / _ItemTabNbLignes[no_item];
                    Uint8 i = 0;
                    y2 = y1 + h;
                    x2=_ItemFondRect[no_item].x + _ItemFondRect[no_item].w;
                    Uint8 i_max = (Uint8) Min( (Uint32)_ItemTabNbLignes[no_item] , (Uint32)_ItemTabValues[no_item].size() );
                    // ceci permet de s'assurer que _ItemTabLigneSelected est slectionn que si a vient de se faire
                    bool selected=false;
                    do{
                        if(y>=y1 && y<=y2){
                            // on surligne la ligne car la souris la survole
                            _ItemTabLigneFond[no_item][i]->setZIndex(Z_MENU_ASCENCEUR);
                            // on enregistre l'indice de la ligne survol afin de la connaitre si le user clique...
                            _ItemTabLigneNumSelected[no_item] = _ItemTabNumFirst[no_item] + i;
                            selected=true;
                            // on indique que la souris survole bien quelque chose
                            on=true;
                        }
                        i++;
                        y1+=h;
                        y2+=h;
                    }while( i < i_max && !on);
                    _ItemTabLigneSelected[no_item] = selected;
                }
    	    }else{ // la liste est vide
                on=false;
                _ItemTabLigneSelected[no_item] = false;
    	    }
        }break;

        default:{
    		x1=_ItemRect[no_item].x - MENU_MARGE_ON;
			// en cas de type input, on permet de cliquer sur toute la zone qui pourra tre saisie
			// soit jusqu' l'extrmit droite de la fentre
			if(_ItemType[no_item] == MENU_TYPE_INPUT){
				x2=_ContourTotal.x+_ContourTotal.w-MENU_FOND_MARGE_G-MENU_FOND_MARGE_D;
			}else{
	    		x2=_ItemRect[no_item].x+_ItemRect[no_item].w+MENU_MARGE_ON+MENU_MARGE_ON;
			}
    		y1=_ItemRect[no_item].y-MENU_MARGE_ON;
    		y2=_ItemRect[no_item].y+_ItemRect[no_item].h+MENU_MARGE_ON+MENU_MARGE_ON;

    		new_font=_ItemFont[no_item];

    		// on met la bonne font suivant si on est sur le texte ou non
    		if(x<x1 || x>x2 || y<y1 || y>y2){
    			new_font=MENU_FONT_ITEM_OUT;
    			on=false;
    		}else{
    			new_font=MENU_FONT_ITEM_ON;
    			on=true;
    		}

    		// si on a chang la font il faut faire le changement dans les sprites
    		if(new_font!=_ItemFont[no_item]){
    			freeItem(no_item);
    			_ItemFont[no_item] = new_font;
    			drawItem(no_item);
    		}
        }break;
	}

	return on;
}

// test si le curseur est sur la fleche retour ou non
bool Menu::testRetourOnOut(Uint16 x, Uint16 y)
{
    if(!_Retour) return false;

     bool on;

     Uint16 x1=_RetourRectDstImg.x - MENU_MARGE_ON;
     Uint16 x2=_RetourRectDstImg.x+_RetourRectDstImg.w+MENU_MARGE_ON+MENU_MARGE_ON;
     Uint16 y1=_RetourRectDstImg.y-MENU_MARGE_ON;
     Uint16 y2=_RetourRectDstImg.y+_RetourRectDstImg.h+MENU_MARGE_ON+MENU_MARGE_ON;

     // on met le bon current suivant si on est sur la fleche ou non
     if(x<x1 || x>x2 || y<y1 || y>y2){
         _AppLineo->_Screen->setSpriteCurrent(_RetourSpriteName,MENU_RETOUR_BLANC);
         _RetourSpriteCurrent=MENU_RETOUR_BLANC;
         on=false;
     }else{
         _AppLineo->_Screen->setSpriteCurrent(_RetourSpriteName,MENU_RETOUR_JAUNE);
         _RetourSpriteCurrent=MENU_RETOUR_JAUNE;
         on=true;
     }

     return on;
}

// Modifie l'image et le texte d'un item liste
void Menu::updateItemListe(Uint8 no_item, string titre_item, string sprite_name)
{
	if(_ItemType[no_item]==MENU_TYPE_LISTE){
		// on supprime l'ancien texte qu'il existe ou non on essaye
	    _AppLineo->_VTexte->del(_ItemTitreId[no_item]);
	    // on met l'image  l'arrire plan afin qu'elle soit cache
	    _AppLineo->_Screen->setSpriteZIndex(_ItemSpriteName[no_item],Z_MIN);
	    // on enregistre les nouveau paramtre de l'item
	    _ItemTitre[no_item] = titre_item;
	    _ItemSpriteName[no_item] = sprite_name;
		// on affiche le nouveau texte si ncessaire
		if(_ItemTitre[no_item]!=ITEM_SANS_TITRE){
            _ItemTitreId[no_item] =
                _AppLineo->_VTexte->add(_ItemFont[no_item],
                                        _ItemTitre[no_item],
                                        (Uint32)(_ItemRect[no_item].x+_ItemRect[no_item].w*0.5-_AppLineo->_VFont->getPhraseWidth(_ItemFont[no_item],_ItemTitre[no_item])*0.5),
                                        _ItemRect[no_item].y+_ItemRect[no_item].h-_AppLineo->_VFont->getHeight(_ItemFont[no_item]),
                                        Z_MENU_TEXTE);
        }
		// on positionne l'image et on la rend visible
		_AppLineo->_Screen->setSpritePosXY(_ItemSpriteName[no_item],_ItemSpriteRect[no_item].x,_ItemSpriteRect[no_item].y);
		_AppLineo->_Screen->setSpriteZIndex(_ItemSpriteName[no_item],Z_MENU_TEXTE);
	}
}

// remplace la liste d'un tableau par une nouvelle
void Menu::updateItemTableau(Uint8 no_item, vector<string> new_tab_values)
{
	if(_ItemType[no_item]==MENU_TYPE_TABLEAU){
		// d'abord on efface le tableau
        freeItem(no_item);
		// on repositionne la liste tout en haut
        _ItemTabNumFirst[no_item]=0;
		//======================================================================
        // on refait tous les calculs avec la nouvelle liste,
		// pour l'ascenceur et le reste...
		//======================================================================
        // on copie la nouvelle liste
        _ItemTabValues[no_item]=new_tab_values;
        // on vrifie que la liste ne soit pas vide car si c'est le cas a simplifi beaucoup de choses...
        if(_ItemTabValues[no_item].size() > 0){
            // on fait des calculs sur l'ascenceur
                // on dfinit s'il y aura besoin d'un ascenceur ou non
            if(_ItemTabValues[no_item].size() > _ItemTabNbLignes[no_item]){
                _ItemTabWithAsc[no_item] = true;
            }else{
                _ItemTabWithAsc[no_item] = false;
            }
            if(_ItemTabWithAsc[no_item]){
                // on calcul la hauteur de l'ascenceur
                _ItemTabAscRect[no_item].h = _ItemRect[no_item].h - MENU_ITEM_TAB_ASC_W - MENU_ITEM_TAB_ASC_W;
                Uint32 min = _ItemTabNbLignes[no_item];
                Uint32 max = _ItemTabValues[no_item].size();
                float reduc = 10.0;
                for(Uint32 i=min;i<max && _ItemTabAscRect[no_item].h > 20;i++){
                    _ItemTabAscRect[no_item].h -= (Uint16)reduc;
                    if(reduc>1.0)reduc-=0.5;
                }
                // on calcul le pas de dplacement de l'ascenceur, un pas correspondant ensuite  une ligne du tableau
                _ItemTabAscPas[no_item] = (float) ((float)(_ItemRect[no_item].h - MENU_ITEM_TAB_ASC_W - MENU_ITEM_TAB_ASC_W - _ItemTabAscRect[no_item].h) / (float)(_ItemTabValues[no_item].size() - _ItemTabNbLignes[no_item]));

            }
            // on initialise les ids qui grerons les lignes de texte  l'cran
            _ItemTabValuesId[no_item].clear();
            Uint32 size = _ItemTabValues->size();
            for(Uint8 i=0;i<size;i++){
                Uint32 id = 0;
                _ItemTabValuesId[no_item].push_back(id);
            }
        }else{ // on a donc => _ItemTabValues[no_item].size() == 0 soit une liste vide
            // il n'y aura donc pas d'ascenceur
            _ItemTabWithAsc[no_item] = false;
            // on initialise les ids qui grerons les lignes de texte  l'cran
            _ItemTabValuesId[no_item].clear();
        }
        // puis on termine en redescinant la liste !
        drawItem(no_item);
	}
}

void Menu::refreshCurseur(void)
{
    if ( _NoItemInputOn < MENU_NB_MAX_ITEM) {
        // on dessine le curseur ou non
        string curseur = (_FlagCurseurClignotant ? "_" : "") ;
        // on supprime l'ancien texte de l'cran curseur compris
        _AppLineo->_VTexte->del(_ItemTitreId[_NoItemInputOn]);
        // on affiche le nouveau texte  gauche + la valeur + le curseur qu'il existe ou non
        if(_ItemIsPassword[_NoItemInputOn]){
            // c'est un champs de saisie d'un mot de passe ou quelques chose qui doit tre cach
            // on remplace donc le texte saisi par le caractre #
            Uint8 size = (*_ItemInputValue[_NoItemInputOn]).size();
            string password = "";
            for(Uint8 i=0;i<size;i++){
                password+="#";
            }
            _ItemTitreId[_NoItemInputOn] = _AppLineo->_VTexte->add(_ItemFont[_NoItemInputOn],_ItemTitre[_NoItemInputOn]+password+curseur,TEXTE_X_LEFT,_ItemRect[_NoItemInputOn].x,_ItemRect[_NoItemInputOn].y,Z_MENU_TEXTE);
            _ItemRect[_NoItemInputOn].w = _AppLineo->_VFont->getPhraseWidth(_ItemFont[_NoItemInputOn],_ItemTitre[_NoItemInputOn]+password+curseur);
        }else{
            // c'est un simple champs de saisie
            _ItemTitreId[_NoItemInputOn] = _AppLineo->_VTexte->add(_ItemFont[_NoItemInputOn],_ItemTitre[_NoItemInputOn]+*_ItemInputValue[_NoItemInputOn]+curseur,TEXTE_X_LEFT,_ItemRect[_NoItemInputOn].x,_ItemRect[_NoItemInputOn].y,Z_MENU_TEXTE);
            _ItemRect[_NoItemInputOn].w = _AppLineo->_VFont->getPhraseWidth(_ItemFont[_NoItemInputOn],_ItemTitre[_NoItemInputOn]+*_ItemInputValue[_NoItemInputOn]+curseur);
        }
    }
}

// gre les vnements du menu
void Menu::processEvents(SDL_Event *event)
{
    bool on = false;

    switch (event->type) {
		// la souris vient de bouger on vrifie si on est sur une zone cliquable du menu
		// afin de modifier l'apparence du texte
        case SDL_MOUSEMOTION:
            for(int i=0;i<_NbItem;i++){
                if(testItemOnOut(i, event->motion.x, event->motion.y)){
                    on = true;
                }
            }
            if(testRetourOnOut(event->motion.x, event->motion.y)){
                on = true;
            }
            // on adapte la forme du curseur
            if(on){
                _AppLineo->_Curseur->setCurseur(CUR_MAIN);
            }else{
                _AppLineo->_Curseur->setCurseur(CUR_FLECHE);
            }
		break;

    	case SDL_MOUSEBUTTONUP:
            if(event->button.button==SDL_BUTTON_LEFT){
                _MouseLeftIsDown = false;
                // si on clique sur un item
                for(int i=0;i<_NbItem;i++){
    				switch(_ItemType[i]){
                        case MENU_TYPE_LISTE:
                            if(_ItemFlecheFontL[i]==MENU_FONT_FLECHE_ON){
        					    _AppLineo->_Events->addEvent(_ItemEventCode[i],&_EventDataFlecheL);
        					}else if(_ItemFlecheFontR[i]==MENU_FONT_FLECHE_ON){
                                _AppLineo->_Events->addEvent(_ItemEventCode[i],&_EventDataFlecheR);
        					}
    					break;

                        case MENU_TYPE_TABLEAU:
                            if(_ItemTabLigneSelected[i]){
                                // on enregistre la valeur
                                _ItemTabValueSelected[i] = _ItemTabValues[i][_ItemTabLigneNumSelected[i]];
                                // on appel l'evnement
                                _AppLineo->_Events->addEvent(_ItemEventCode[i],&_ItemTabValueSelected[i]);
                            }
                        break;

                        case MENU_TYPE_INPUT:
                            if(_ItemFont[i]==MENU_FONT_ITEM_ON){
                                setItemOn(i);
                                _AppLineo->_Curseur->setCurseur(CUR_FLECHE);
                            }
                        break;

                        default:
                            if(_ItemFont[i]==MENU_FONT_ITEM_ON){
                                if(_ItemEventData[i]==MENU_EVENT_NO_DATA){
                                    _AppLineo->_Events->addEvent(_ItemEventCode[i]);
                                }else{
                                    _AppLineo->_Events->addEvent(_ItemEventCode[i],&_ItemEventData[i]);
                                }
                                _AppLineo->_Curseur->setCurseur(CUR_FLECHE);
        					}
        				break;
    				}
                }
                // si on clique sur la fleche retour
                if(_Retour){
                    if(_RetourSpriteCurrent==MENU_RETOUR_JAUNE){
                        _AppLineo->_Events->addEvent(_RetourEventCode);
                        _AppLineo->_Curseur->setCurseur(CUR_FLECHE);
                    }
                }
            }
        break;

    	case SDL_MOUSEBUTTONDOWN:
            if(event->button.button==SDL_BUTTON_LEFT){
                _MouseLeftIsDown = true;
                // on cherche s'il y a des tableaux auquel cas on peut avoir des actions...
                for(int i=0;i<_NbItem;i++){
                    if(_ItemType[i]==MENU_TYPE_TABLEAU){
                        if(_ItemTabWithAsc[i]){
                            if(_ItemTabBtBas[i]){
                                itemTabBtBasPressed(i);
                            }else if(_ItemTabBtHaut[i]){
                                itemTabBtHautPressed(i);
                            }
                        }
                    }
                }
            }else if(event->button.button==MOUSE_MOLETTE_UP){
                // si on est au dessus du tableau on le fait dfiler vers le haut 3 fois
                for(int i=0;i<_NbItem;i++){
                    if(_ItemType[i]==MENU_TYPE_TABLEAU
                    && _ItemTabWithAsc[i]
                    && _ItemTabLigneSelected[i])
                    {
                        itemTabBtHautPressed(i);
                        itemTabBtHautPressed(i);
                        itemTabBtHautPressed(i);
                    }
                }
            }else if(event->button.button==MOUSE_MOLETTE_DOWN){
                // si on est au dessus du tableau on le fait dfiler vers le bas 3 fois
                for(int i=0;i<_NbItem;i++){
                    if(_ItemType[i]==MENU_TYPE_TABLEAU
                    && _ItemTabWithAsc[i]
                    && _ItemTabLigneSelected[i])
                    {
                        itemTabBtBasPressed(i);
                        itemTabBtBasPressed(i);
                        itemTabBtBasPressed(i);
                    }
                }
            }
        break;

        case SDL_USEREVENT:
            switch (event->user.code) {
                case EVENT_TIMER:
                    // on fait clignoter le curseur si un item input est slectionn
                    if(_NoItemInputOn < MENU_NB_MAX_ITEM) {
                        _FlagCurseurClignotant = (_FlagCurseurClignotant ? false : true) ;
                        refreshCurseur();
                    }
                    // on poursuit le dfilement d'un tableau si le bouton de la souris est toujours enfonc
                    if(_MouseLeftIsDown){
                        // on cherche s'il y a des tableaux et s'ils sont modifiables avec un bouton actif
                        for(int i=0;i<_NbItem;i++){
                            if(_ItemType[i]==MENU_TYPE_TABLEAU && _ItemTabWithAsc[i]){
                                if(_ItemTabWithAsc[i]){
                                    if(_ItemTabBtBas[i]){
                                        itemTabBtBasPressed(i);
                                    }else if(_ItemTabBtHaut[i]){
                                        itemTabBtHautPressed(i);
                                    }
                                }
                            }
                        }
                    }
                break;
            }
        break;

        // Si on appuie sur une touche
        case SDL_KEYDOWN:{
            for(int no_item=0; no_item < _NbItem; no_item++){
                if ( (_ItemType[no_item]==MENU_TYPE_INPUT) && (_ItemTypeValue[no_item]==MENU_TYPE_INPUT_ON) ){
                // Si on efface un caractre
                    if(event->key.keysym.sym==SDLK_BACKSPACE){
						if ((*_ItemInputValue[no_item]).size() != 0){
							(*_ItemInputValue[no_item]).erase((*_ItemInputValue[no_item]).size()-1,1);
						}
						_FlagCurseurClignotant=true;
						refreshCurseur();
						_AppLineo->_Events->addEvent(_ItemEventCode[no_item]);
                    }
               // Si on ajoute un caractre
                    else if (event->key.keysym.unicode > ASCII_SPACE){
						if ((*_ItemInputValue[no_item]).size() <= 17){
							*_ItemInputValue[no_item] += event->key.keysym.unicode;
						}
						_FlagCurseurClignotant=true;
						refreshCurseur();
						_AppLineo->_Events->addEvent(_ItemEventCode[no_item]);
                    }else if(event->key.keysym.sym >= SDLK_KP0 && event->key.keysym.sym <= SDLK_KP9){
						switch(event->key.keysym.sym){
							case SDLK_KP0: *_ItemInputValue[no_item] += "0"; break;
							case SDLK_KP1: *_ItemInputValue[no_item] += "1"; break;
							case SDLK_KP2: *_ItemInputValue[no_item] += "2"; break;
							case SDLK_KP3: *_ItemInputValue[no_item] += "3"; break;
							case SDLK_KP4: *_ItemInputValue[no_item] += "4"; break;
							case SDLK_KP5: *_ItemInputValue[no_item] += "5"; break;
							case SDLK_KP6: *_ItemInputValue[no_item] += "6"; break;
							case SDLK_KP7: *_ItemInputValue[no_item] += "7"; break;
							case SDLK_KP8: *_ItemInputValue[no_item] += "8"; break;
							case SDLK_KP9: *_ItemInputValue[no_item] += "9"; break;
						}
						_FlagCurseurClignotant=true;
						refreshCurseur();
						_AppLineo->_Events->addEvent(_ItemEventCode[no_item]);
					}
                }
            }
        }break;

    	default:
	    break;
	}
}

// on dessine un ascenceur  droite du MENU_TYPE_TABLEAU
void Menu::drawAscenceur(Uint8 no_item)
{
    SDL_Rect src,des;
    // on cre le sprite
    _AppLineo->_Screen->addSprite(_ItemSpriteName[no_item],(string)(DATA_PATH)+"menus/ascenceur.png",Z_MENU_ASCENCEUR,true);
    // on ajoute le fond de l'ascenceur
    src.x=0;
    src.y=0;
    src.w=16;
    src.h=_ItemFondRect[no_item].h;
    des.x = _ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
    des.y = _ItemFondRect[no_item].y;
    _AppLineo->_Screen->addSpriteImage(_ItemSpriteName[no_item],src,des);
    // l'apparence des boutons (et de l'ascenceur...) dpend si on affihe l'ascenceur ou non
    if(_ItemTabWithAsc[no_item]){
        // on ajoute le bouton du haut
        src.x=17;
        src.y=0;
        src.w=src.h=16;
        des.x = _ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
        des.y = _ItemFondRect[no_item].y;
        _AppLineo->_Screen->addSpriteImage(_ItemSpriteName[no_item],src,des);
        // on ajoute le bouton du bas
        src.x=17;
        src.y=17;
        src.w=src.h=16;
        des.x = _ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
        des.y = _ItemFondRect[no_item].y + _ItemFondRect[no_item].h - MENU_ITEM_TAB_ASC_W;
        _AppLineo->_Screen->addSpriteImage(_ItemSpriteName[no_item],src,des);
        // on calcul le dcalage max pour que l'ascenceur ne soit pas trop bas,
        // car il ne faudrait pas  cause des arrondie qu'il passe par dessus le bouton du bas
        Uint16 decalage_max = _ItemRect[no_item].h - MENU_ITEM_TAB_ASC_W - MENU_ITEM_TAB_ASC_W - _ItemTabAscRect[no_item].h;
        // on ajoute l'ascenceur lui mme
        Uint16 decalage_y = (Uint16)( ((float)(_ItemTabNumFirst[no_item])) * _ItemTabAscPas[no_item] );
        src.x=17;
        src.y=68;
        src.w=16;
        src.h=_ItemTabAscRect[no_item].h - 8;
        des.x = _ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
        des.y = _ItemFondRect[no_item].y + MENU_ITEM_TAB_ASC_W + Min(decalage_y,decalage_max);
        _AppLineo->_Screen->addSpriteImage(_ItemSpriteName[no_item],src,des);
        src.x=17;
        src.y=440;
        src.w=16;
        src.h=8;
        des.x = _ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
        des.y = _ItemFondRect[no_item].y + MENU_ITEM_TAB_ASC_W + _ItemTabAscRect[no_item].h - 8 + decalage_y;
        _AppLineo->_Screen->addSpriteImage(_ItemSpriteName[no_item],src,des);
    }else{
        // on ajoute le bouton du haut
        src.x=17;
        src.y=34;
        src.w=src.h=16;
        des.x = _ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
        des.y = _ItemFondRect[no_item].y;
        _AppLineo->_Screen->addSpriteImage(_ItemSpriteName[no_item],src,des);
        // on ajoute le bouton du bas
        src.x=17;
        src.y=34 + 17;
        src.w=src.h=16;
        des.x = _ItemFondRect[no_item].x + _ItemFondRect[no_item].w + 2;
        des.y = _ItemFondRect[no_item].y + _ItemFondRect[no_item].h - MENU_ITEM_TAB_ASC_W;
        _AppLineo->_Screen->addSpriteImage(_ItemSpriteName[no_item],src,des);
    }
}

void Menu::itemTabBtBasPressed(Uint8 no_item)
{
    if(_ItemTabNumFirst[no_item] < _ItemTabValues[no_item].size() - _ItemTabNbLignes[no_item]){
        freeItem(no_item);
        _ItemTabNumFirst[no_item]++;
        drawItem(no_item);
    }
}


void Menu::itemTabBtHautPressed(Uint8 no_item)
{
    if(_ItemTabNumFirst[no_item] > 0){
        freeItem(no_item);
        _ItemTabNumFirst[no_item]--;
        drawItem(no_item);
    }
}
