//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "MiniPiece.h"
#include "Lineo.h"

// on initialise toutes les donnes suivant la pice qui est indique
MiniPiece::MiniPiece(Lineo* app_lineo, Uint8 forme)
{
    static Uint32 indice=0;

	_AppLineo = app_lineo;
    _Forme = forme;

    _Selected = false;
    _OnScreen = false;

    string color;
    if(_Forme & P_FORME_COLOR_1){
        color = _AppLineo->_Config->get("couleur1");
    }else{
        color = _AppLineo->_Config->get("couleur2");
    }
    _SurfaceName = (string)(DATA_PATH)+"game/pieces/"+color+"_2d.png";

    _SurfaceNameCont = (string)(DATA_PATH)+"game/pieces/contour_2d.png";

    // grce  l'indice qui est unique (car static) le nom du sprite le sera galement
    _SpriteName = "mini_piece_" + to_string(indice++);
    _SpriteNameCont = "mini_piece_contour_" + to_string(indice++);

    // on formate plusieurs paramtres pour les 16 pices
    if(_Forme & P_FORME_ROND && _Forme & P_FORME_PETITE && _Forme & P_FORME_CREUSE){
        _RecSrc.x=138;
        _RecSrc.y=18;
        _RecSrc.w=17;
        _RecSrc.h=17;
        _RecSrcCont.x=132;
        _RecSrcCont.y=12;
        _RecSrcCont.w=29;
        _RecSrcCont.h=29;
    }else if(_Forme & P_FORME_ROND && _Forme & P_FORME_PETITE && _Forme & P_FORME_PLEINE){
        _RecSrc.x=138;
        _RecSrc.y=58;
        _RecSrc.w=17;
        _RecSrc.h=17;
        _RecSrcCont.x=132;
        _RecSrcCont.y=52;
        _RecSrcCont.w=29;
        _RecSrcCont.h=29;
    }else if(_Forme & P_FORME_ROND && _Forme & P_FORME_GRANDE && _Forme & P_FORME_CREUSE){
        _RecSrc.x=93;
        _RecSrc.y=13;
        _RecSrc.w=27;
        _RecSrc.h=27;
        _RecSrcCont.x=87;
        _RecSrcCont.y=7;
        _RecSrcCont.w=39;
        _RecSrcCont.h=39;
    }else if(_Forme & P_FORME_ROND && _Forme & P_FORME_GRANDE && _Forme & P_FORME_PLEINE){
        _RecSrc.x=93;
        _RecSrc.y=53;
        _RecSrc.w=27;
        _RecSrc.h=27;
        _RecSrcCont.x=87;
        _RecSrcCont.y=47;
        _RecSrcCont.w=39;
        _RecSrcCont.h=39;
    }else if(_Forme & P_FORME_CARRE && _Forme & P_FORME_PETITE && _Forme & P_FORME_CREUSE){
        _RecSrc.x=58;
        _RecSrc.y=18;
        _RecSrc.w=17;
        _RecSrc.h=17;
        _RecSrcCont.x=52;
        _RecSrcCont.y=12;
        _RecSrcCont.w=29;
        _RecSrcCont.h=29;
    }else if(_Forme & P_FORME_CARRE && _Forme & P_FORME_PETITE && _Forme & P_FORME_PLEINE){
        _RecSrc.x=58;
        _RecSrc.y=58;
        _RecSrc.w=17;
        _RecSrc.h=17;
        _RecSrcCont.x=52;
        _RecSrcCont.y=52;
        _RecSrcCont.w=29;
        _RecSrcCont.h=29;
    }else if(_Forme & P_FORME_CARRE && _Forme & P_FORME_GRANDE && _Forme & P_FORME_CREUSE){
        _RecSrc.x=13;
        _RecSrc.y=13;
        _RecSrc.w=27;
        _RecSrc.h=27;
        _RecSrcCont.x=7;
        _RecSrcCont.y=7;
        _RecSrcCont.w=39;
        _RecSrcCont.h=39;
    }else if(_Forme & P_FORME_CARRE && _Forme & P_FORME_GRANDE && _Forme & P_FORME_PLEINE){
        _RecSrc.x=13;
        _RecSrc.y=53;
        _RecSrc.w=27;
        _RecSrc.h=27;
        _RecSrcCont.x=7;
        _RecSrcCont.y=47;
        _RecSrcCont.w=39;
        _RecSrcCont.h=39;
    }
    // on rgle l'offset d'affichage de la pice en fonction de sa taille
    if(_Forme & P_FORME_GRANDE){
        _OffsetX=MINIP_OFFSET_GRANDE_X;
        _OffsetY=MINIP_OFFSET_GRANDE_Y;
        _OffsetXCont=MINIP_OFFSET_CONTOUR_GRANDE_X;
        _OffsetYCont=MINIP_OFFSET_CONTOUR_GRANDE_Y;
    }else{
        _OffsetX=MINIP_OFFSET_PETITE_X;
        _OffsetY=MINIP_OFFSET_PETITE_Y;
        _OffsetXCont=MINIP_OFFSET_CONTOUR_PETITE_X;
        _OffsetYCont=MINIP_OFFSET_CONTOUR_PETITE_Y;
    }
}

MiniPiece::~MiniPiece()
{
	freeScreen();
}

// dessine la pices dans la classe screen
void MiniPiece::drawScreen(Uint16 x, Uint16 y, Uint32 z_index)
{
    _PosPointX = x;
    _PosPointY = y;

    _ZIndex = z_index;

    _RecDes.x = _PosPointX - _OffsetX;
    _RecDes.y = _PosPointY - _OffsetY;

    _RecDes.w = _RecSrc.w;
    _RecDes.h = _RecSrc.h;

    _AppLineo->_Screen->addSprite(_SpriteName,_SurfaceName,_ZIndex);
    _AppLineo->_Screen->addSpriteImage(_SpriteName,_RecSrc,_RecDes);

    _OnScreen = true;
}

// supprime la pices de la classe screen
void MiniPiece::freeScreen(void)
{
    _AppLineo->_Screen->delSprite(_SpriteName);
    _AppLineo->_Screen->delSprite(_SpriteNameCont);

    _OnScreen = false;
}

// place ou enlve un contour autour de la pice
void MiniPiece::setSelected(bool selected)
{
    if(_Selected!=selected){
        _Selected=selected;
        if(_Selected==true && _OnScreen==true){
            _RecDesCont.x = _PosPointX - _OffsetXCont;
            _RecDesCont.y = _PosPointY - _OffsetYCont;

            _RecDesCont.w = _RecSrcCont.w;
            _RecDesCont.h = _RecSrcCont.h;

            _AppLineo->_Screen->addSprite(_SpriteNameCont,_SurfaceNameCont,_ZIndex-1);
            _AppLineo->_Screen->addSpriteImage(_SpriteNameCont,_RecSrcCont,_RecDesCont);
        }else{
            _AppLineo->_Screen->delSprite(_SpriteNameCont);
        }
    }
}
