//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef __SURFACE_H__
#define __SURFACE_H__

#include <string>
#include <SDL.h>
#include <SDL_image.h>

using namespace std;

// Gre une surface :
// > la charge en mmoire
// > lui attribut un pointeur SDL_Surface
// > connat le nombre de sprite qui lui sont attachs
// > la supprime de la mmoire
class Surface
{
  //*******************
  //* variables membres
    public:
        Uint16 _NbSprite; // dfinit le nombre de sprite utilisant cette image, a permet de supprimer l'image si ==0
        
	private:
        Uint32 _Index; // index unique pour chaque surface, permet de la pointer rapidement, correspond  l'index du vector
        SDL_Surface* _Surface; // pointeur sur l'image charge en mmoire
        string _Name; // chemin et nom du fichier  charger dans l'image, doit tre unique pour chaque instancation de la classe
        

  //*******************
  //* fonctions membres
	public:
        Surface(string name); 
        Surface(string name, Uint32 index); 
		~Surface(); 
		void addSprite(); // incrment le nombre de sprite
		void delSprite(); // dcrmente le nombre de sprite
		Uint8 getPixelAlpha(Uint16 x, Uint16 y); // donne la valeur apha du pixel (x,y) de la surface
		// Oprateur d'galit :
        bool operator==(const Surface &surface) const
        {
            return (_Name == surface._Name);
        }
        SDL_Surface* getSurface()
		{
			return _Surface;
		}
        Uint32 getIndex()
		{
			return _Index;
		}
	private:
		void loadSurface(); // charge une surface
		void freeSurface(); // libre une surface
		Uint32 getpixel(Uint16 x, Uint16 y); // retourne la valeur d'un pixel au point x,y
};


#endif // __SURFACE_H__
