//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VueChoixCouleurPiece.h"
#include "Lineo.h"

VueChoixCouleurPiece::VueChoixCouleurPiece(Lineo* app_lineo)
{
	_AppLineo = app_lineo;
	// on initialise la liste des couleurs
	_ListeCouleurs.clear();
	_ListeCouleurs.push_back("violet");
	_ListeCouleurs.push_back("vert");
	_ListeCouleurs.push_back("jaune");
	_ListeCouleurs.push_back("bleu");
	_ListeCouleurs.push_back("orange");
	_ListeCouleurs.push_back("rose");
	_ListeCouleurs.push_back("rouge");
	// on initialise les images des pices, en listant toutes les couleurs
	// les dimensions sont les mme  chaque fois on ne les calcul donc qu'une fois
	// P_FORME_CARRE | P_FORME_GRANDE | P_FORME_CREUSE
	// on a besoin que d'une image par couleur car elle ne peut tre en haut et en bas  la fois
    SDL_Rect RecDes, RecSrc;
    RecSrc.x=76; RecSrc.y=74;
    RecSrc.w=41; RecSrc.h=81;
	RecDes.x = RecDes.y = 0;
    RecDes.w = RecSrc.w; RecDes.h = RecSrc.h;
	int liste_size=_ListeCouleurs.size();
    for(int i=0;i<liste_size;i++){
	    _AppLineo->_Screen->addSprite(_ListeCouleurs[i],(string)(DATA_PATH)+"game/pieces/"+_ListeCouleurs[i]+"_3d.png",Z_MIN);
    	_AppLineo->_Screen->addSpriteImage(_ListeCouleurs[i],RecSrc,RecDes);
    }
    // on initialise le fond d'cran
    _FondEcran = new FondEcran(_AppLineo, FD_ECR_VIERGE|FD_ECR_LINEO_SMALL);
    // on initialise le menu
    _Menu = new Menu(_AppLineo, _AppLineo->_Langue->get("Couleurs des pieces"), true, EVENT_CREATE_VUE_OPTION);
	_ListeCoulIter1=findCouleur(_AppLineo->_Config->get("couleur1"));
    _NoItemColor1 = _Menu->addItem(ITEM_SANS_TITRE,EVENT_CHOIX_COLOR_PC_1,MENU_TYPE_LISTE,MENU_EVENT_NO_DATA,*_ListeCoulIter1,RecSrc.w,RecSrc.h);
	_ListeCoulIter2=findCouleur(_AppLineo->_Config->get("couleur2"));
    _NoItemColor2 = _Menu->addItem(ITEM_SANS_TITRE,EVENT_CHOIX_COLOR_PC_2,MENU_TYPE_LISTE,MENU_EVENT_NO_DATA,*_ListeCoulIter2,RecSrc.w,RecSrc.h);
    _Menu->draw();
}

VueChoixCouleurPiece::~VueChoixCouleurPiece()
{
    // on efface les pices
	int liste_size=_ListeCouleurs.size();
    for(int i=0;i<liste_size;i++){
	    _AppLineo->_Screen->delSprite(_ListeCouleurs[i]);
    }
    // on libre les pointeurs des classes
    delete(_Menu);
	delete(_FondEcran);
}

// permet d'aller  l'itrateur suivant, ou retourner au premier si le dernier est atteind
vstring_iter VueChoixCouleurPiece::listeCoulNext(vstring_iter current_iter)
{
	current_iter++;
	if(current_iter==_ListeCouleurs.end()){
        current_iter = _ListeCouleurs.begin();
	}
	return current_iter;
}

// permet d'aller  l'itrateur prcdent, ou retourner au dernier si le premier est atteind
vstring_iter VueChoixCouleurPiece::listeCoulPrev(vstring_iter current_iter)
{
	if(current_iter==_ListeCouleurs.begin()){
        current_iter = _ListeCouleurs.end();
	}
	current_iter--;
	return current_iter;
}

// gre les vnements de la vue
void VueChoixCouleurPiece::processEvents(SDL_Event *event)
{
    // on s'occupe de notre vue en premier
    switch (event->type) {
        case SDL_USEREVENT:
            switch (event->user.code)
            {
                case EVENT_CHOIX_COLOR_PC_1:{
                    Uint16 fleche = *(Uint16*)event->user.data1;
                    if(fleche==MENU_FLECHE_L){
                 		_ListeCoulIter1 = listeCoulPrev(_ListeCoulIter1);
                 		if(_ListeCoulIter1==_ListeCoulIter2){
							_ListeCoulIter1 = listeCoulPrev(_ListeCoulIter1);
						}
                 		_Menu->updateItemListe(_NoItemColor1,ITEM_SANS_TITRE,*_ListeCoulIter1);
					}else if(fleche==MENU_FLECHE_R){
                 		_ListeCoulIter1 = listeCoulNext(_ListeCoulIter1);
                 		if(_ListeCoulIter1==_ListeCoulIter2){
							_ListeCoulIter1 = listeCoulNext(_ListeCoulIter1);
						}
                 		_Menu->updateItemListe(_NoItemColor1,ITEM_SANS_TITRE,*_ListeCoulIter1);
					}
					_AppLineo->_Config->set("couleur1",(*_ListeCoulIter1));
                }break;
                case EVENT_CHOIX_COLOR_PC_2:{
                    Uint16 fleche = *(Uint16*)event->user.data1;
                    if(fleche==MENU_FLECHE_L){
                 		_ListeCoulIter2 = listeCoulPrev(_ListeCoulIter2);
                 		if(_ListeCoulIter2==_ListeCoulIter1){
							_ListeCoulIter2 = listeCoulPrev(_ListeCoulIter2);
						}
                 		_Menu->updateItemListe(_NoItemColor2,ITEM_SANS_TITRE,*_ListeCoulIter2);
					}else if(fleche==MENU_FLECHE_R){
                 		_ListeCoulIter2 = listeCoulNext(_ListeCoulIter2);
                 		if(_ListeCoulIter2==_ListeCoulIter1){
							_ListeCoulIter2 = listeCoulNext(_ListeCoulIter2);
						}
                 		_Menu->updateItemListe(_NoItemColor2,ITEM_SANS_TITRE,*_ListeCoulIter2);
					}
					_AppLineo->_Config->set("couleur2",(*_ListeCoulIter2));
                }break;
            }
        break;
   	}
   	// puis du menu
    _Menu->processEvents(event);
}

// rcupre l'itrateur d'une couleur dans la liste
// si on trouve pas la couleur on renvois la premire
vstring_iter VueChoixCouleurPiece::findCouleur(string couleur)
{
    vstring_iter itRecherche = find(_ListeCouleurs.begin(), _ListeCouleurs.end(), couleur);
	if (itRecherche != _ListeCouleurs.end())
    {
		return itRecherche;
	}
	return _ListeCouleurs.begin();
}
