//  Lino
//  Copyright (C) 2006 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//  Copyright (C) 2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#ifndef __VUE_LOGIN_H__
#define __VUE_LOGIN_H__

#include <math.h>
#include <SDL_thread.h>


#include "Screen.h"
#include "Menu.h"
#include "Config.h"
#include "FondEcran.h"
#include "Langue.h"
#include "ComReseau.h"

// =============================================================================
// Copyright (C) 2005 Quentin Carbonneaux : GNU General Public License
// macro convertissant une partie de chaine en Uint32 en little-endian
// elles simplifient les calculs du md5
#define GET_UINT32(a,b,i)						\
{												\
	(a) = ( (unsigned int) (b)[(i)	]      )	\
	    | ( (unsigned int) (b)[(i)+1] << 8 )	\
	    | ( (unsigned int) (b)[(i)+2] << 16)	\
	    | ( (unsigned int) (b)[(i)+3] << 24);	\
}
// Rotation  gauche d'une valeur sur 32 bits (mot)
#define ROTATE_LEFT(w,s) ((w << s) | ((w & 0xFFFFFFFF) >> (32 - s)))
// Fonctions d'encodage md5
#define F(x,y,z) ((x & y) | (~x & z))
#define G(x,y,z) ((x & z) | (~z & y))
#define H(x,y,z) (x ^ y ^ z)
#define I(x,y,z) (y ^ (x | ~z))
#define FF(a,b,c,d,x,s,t) (a = b + ROTATE_LEFT((a + F(b,c,d) + x + t), s))
#define GG(a,b,c,d,x,s,t) (a = b + ROTATE_LEFT((a + G(b,c,d) + x + t), s))
#define HH(a,b,c,d,x,s,t) (a = b + ROTATE_LEFT((a + H(b,c,d) + x + t), s))
#define II(a,b,c,d,x,s,t) (a = b + ROTATE_LEFT((a + I(b,c,d) + x + t), s))
// les valeurs avec sin sont prcalcule, on gagne du temps et de la prcision
#define S11 7
#define S12 12
#define S13 17
#define S14 22
#define S21 5
#define S22 9
#define S23 14
#define S24 20
#define S31 4
#define S32 11
#define S33 16
#define S34 23
#define S41 6
#define S42 10
#define S43 15
#define S44 21
// =============================================================================

class Lineo;

using namespace std;

typedef unsigned char uchar;

class VueLogin
{
  //*******************
  //* variables membres
	private:
	    Lineo 		*_AppLineo;
        Menu		*_Menu;
        FondEcran	*_FondEcran;
        ComReseau   *_ComReseau;
        SDL_Thread 	*_Thread;
        
        Uint8   _LoginItem;
        string  _Login;
        Uint8   _PassItem;
        string  _Pass;


  //*******************
  //* fonctions membres        
	public:
		VueLogin(Lineo* app_lineo);
		~VueLogin();
		void processEvents(SDL_Event *event);

    private:
        static int staticLoginThread( void *Param ){
            VueLogin* Obj = reinterpret_cast<VueLogin*>( Param );
            return Obj->LoginThread(Param);
        }
        int LoginThread( void *Param );
        string getMD5(string pass);
};

#endif // __VUE_LOGIN_H__
